'use strict';
const utility = require("./util");
function createReduct(parent) {
    let parentInjector;
    if (parent instanceof Map) {
        parentInjector = (key) => parent.get(key);
    }
    else if (typeof parent === 'object') {
        parentInjector = (key) => parent[key.name];
    }
    else if (typeof parent === 'function') {
        parentInjector = parent;
    }
    else if (typeof parent === 'undefined') {
        parentInjector = () => false;
    }
    else {
        throw new TypeError('Parent injector must be a Map, object or function');
    }
    const cache = new Map();
    let stack = new Set();
    let queue = [];
    const reduct = (Constructor) => {
        if (typeof Constructor !== 'function') {
            throw new TypeError('Dependencies must be constructors/factories, but got: ' + typeof Constructor);
        }
        if (stack.has(Constructor)) {
            const stackArray = utility.convertSetToArray(stack);
            stackArray.push(Constructor);
            const prettyStack = stackArray.map(utility.printPrettyConstructor).join(' => ');
            throw new Error('Circular dependency detected: ' + prettyStack);
        }
        stack.add(Constructor);
        const instance = cache.get(Constructor) ||
            parentInjector(Constructor) ||
            new Constructor(reduct);
        stack.delete(Constructor);
        cache.set(Constructor, instance);
        if (queue.length) {
            stack.add(utility.printPrettyConstructor(Constructor) + ' (post)');
            const lastQueue = queue;
            queue = [];
            lastQueue.forEach((fn) => fn());
        }
        return instance;
    };
    reduct.later = (fn) => queue.push(fn);
    return reduct;
}
(function (createReduct) {
    createReduct.util = utility;
})(createReduct || (createReduct = {}));
module.exports = createReduct;
//# sourceMappingURL=index.js.map